;-------------------------------------------------------------------------------
;       Einbinden der controllerspezifischen Definitionsdatei
;-------------------------------------------------------------------------------
.NOLIST                         ;List-Output ausschalten
.INCLUDE "m32def.inc"           ;AVR-Definitionsdatei einbinden
.INCLUDE "reszif.asm"           ;Resevierung Ziffernanzeige
.LIST                           ;List-Output wieder einschalten
;
; MACRO increment Memory WORD
;
.MACRO INCM ; MEM + 1
    push r16
    lds r16,@0
   	inc r16
   	sts @0,r16
    brne PC+6
                 ;
    lds r16,@0+1
   	inc r16
  	sts @0+1,r16
    pop r16
.ENDMACRO
;*******************************************************************************
; alle Parameter von 000 - 899 stehen zur freien Verfgung
;*******************************************************************************
.equ demo_init   = var301      ; 300 Zaehler wird imer nach Reset um 1 erhht
.equ demo_10ms   = var301      ; 301 Zaehler wird alle 10ms um 1 erhht
.equ demo_100ms  = var302      ; 302 Zaehler wird alle 100ms um 1 erhht
.equ demo_1000ms = var303      ; 303 Zaehler wird alle 1000ms um 1 erhht
.equ demo_zyk    = var304      ; 304 Zaehler wird bei jedem Programmzyklus
;                                    um 1 erhht
;
.equ merker1     = var801      ; 801 Merker um Led 1 zu schalten 1 Byte reicht
.equ merker2     = var801+1    ; 801+1 Merker um Led 2 zu schalten 1 Byte reicht
.equ merker3     = var802      ; 802 Merker um Hupe zu schalten 1 Byte reicht
;*******************************************************************************
; alle Parameter von 900 - 999 sind vom System belegt
;*******************************************************************************
;
; sr_inp_portx wird am Anfang eines jedes Zyklus vom Eingangsport eingelesen
; und nur hier
.equ sr_inp_porta = var901   ; 901 Eingangsbyte Port A
.equ sr_inp_portb = var902   ; 902 Eingangsbyte Port B
.equ sr_inp_portc = var903   ; 903 Eingangsbyte Port C
.equ sr_inp_portd = var904   ; 904 Eingangsbyte Port D
;
; sr_oup_portx wird am Ende jedes Zyklus an das Ausgangsport weitergegeben
; und nur hier
.equ sr_oup_porta = var905   ; 905 Ausgangsbyte Port A
.equ sr_oup_portb = var906   ; 905 Ausgangsbyte Port B
.equ sr_oup_portc = var907   ; 905 Ausgangsbyte Port C
.equ sr_oup_portd = var908   ; 908 Eingangsbyte Port D

.equ sr_sec       = var912   ; 912 Sekunde
.equ sr_min       = var913   ; 913 Minute 
.equ sr_std       = var914   ; 914 Stunde
.equ sr_tag       = var915   ; 915 Tag    
.equ sr_mon       = var916   ; 916 Monat  
.equ sr_jahr      = var915   ; 917 Jahr
;
;                            ; 921 SRAM-Daten von 000-499 
;                            ;     und Datum und Uhrzeit
;                            ;     1 = im EEPROM ablegen
;                            ;     2 = aus EEPROM lesen
;                            ;  1001 = alle Werte von 000-499 auf 0 setzen
;                            ;  1002 = alle Werte von 500-899 auf 0 setzen
;
;                            ; 935 Programmzyklen pro Sekunde
;
;*******************************************************************************
	
	
;===============================================================================
.CSEG                                                                            
.ORG    0x1000                  
user_init:    
 nop
jmp initialisieren
;===============================================================================
.CSEG                                                                            
.ORG    0x1004                  
jede_hsekunde:
 nop
jmp alle_10ms
;===============================================================================
.CSEG                                                                            
.ORG    0x1008                  
jede_zsekunde:
 nop
jmp alle_100ms
;===============================================================================
.ORG    0x100c                  
jede_sekunde:
 nop
jmp alle_1000ms
;===============================================================================
.ORG    0x1010                  
jeder_zyklus:
 nop
jmp alle_zyklen
;===============================================================================
;
;*******************************************************************************
; was hier steht wird nach jedem Neustart ausgefhrt
;*******************************************************************************
;
initialisieren:
 incm demo_init
;
; hier knnen die Ports A-C neu initialisiert werden.
; Bitte Port D nicht ndern.
; z.B.
         ldi r16, 0x00
         out DDRA, r16     ; Alle Pins am Port A durch Ausgabe von 0x00 in
                           ; Richtungsregister DDRA als Eingang konfigurieren
         ldi r16, 0xFF
         out DDRB, r16     ; Alle Pins am Port B durch Ausgabe von 0xFF ins
                           ; Richtungsregister DDRB als Ausgang konfigurieren
         ldi r16, 0xFF
         out DDRC, r16     ; Alle Pins am Port C durch Ausgabe von 0xFF ins
                           ; Richtungsregister DDRC als Ausgang konfigurieren
 ret
;
;*******************************************************************************
; was hier steht wird alle 10ms ausgefhrt
;*******************************************************************************
;
alle_10ms:
 incm demo_10ms
 ret
;
;*******************************************************************************
; was hier steht wird alle 100ms ausgefhrt
;*******************************************************************************
;
alle_100ms:
 incm demo_100ms
 call Led1_toggeln
 ret
;
;*******************************************************************************
; was hier steht wird alle 1000ms ausgefhrt
;*******************************************************************************
;
alle_1000ms:
 incm demo_1000ms
 call Led2_toggeln
 ret
;
;*******************************************************************************
; was hier steht was so den Rest der Zeit ausgefhrt werden soll
; das heit alle Interrupts sind abgearbeitet
;*******************************************************************************
;
alle_zyklen:
 incm demo_zyk
 call taster3_gdr
 call im_kreis_laufen 
 ret
;-------------------------------------------------------------------------------
;	Led1 toggeln  
;-------------------------------------------------------------------------------
;  
.equ Led1 = 32
Led1_toggeln:
    lds r17,merker1
	cpi r17,0
	brne end_led
;
    lds r16,sr_oup_portd
	ori r16,Led1
    sts sr_oup_portd,r16
	ldi r17,1
	sts merker1,r17
    ret
;
;
end_led:
    lds r16,sr_oup_portd
	andi r16,(255-Led1)
    sts sr_oup_portd,r16
    ldi r17,0
	sts merker1,r17
    ret
;-------------------------------------------------------------------------------
;	Led2 toggeln  
;-------------------------------------------------------------------------------
;  
.equ Led2 = 64
Led2_toggeln:
    lds r17,merker2
	cpi r17,0
	brne end_led2
;
    lds r16,sr_oup_portd
	ori r16,Led2
    sts sr_oup_portd,r16
	ldi r17,1
	sts merker2,r17
    ret
;
;
end_led2:
    lds r16,sr_oup_portd
	andi r16,(255-Led2)
    sts sr_oup_portd,r16
    ldi r17,0
	sts merker2,r17
    ret
;-------------------------------------------------------------------------------
;
;	Taster 3 abfragen und solange wie gedrckt hupen
;
;-------------------------------------------------------------------------------
.equ Taster3 = 0x10
; sr_inp_portd wird am Anfang jedes Zykluses aktualisiert
taster3_gdr:
    lds r16,sr_inp_portd
	andi r16,Taster3
 	breq k_taste3
;
    call Hupe_toggeln
k_taste3:
    ret

;-------------------------------------------------------------------------------
;	Hupe toggeln
;-------------------------------------------------------------------------------
;  
.equ Hupe = 128
Hupe_toggeln:
    lds r17,merker3
	cpi r17,0
	brne end_hupe
;
    lds r16,sr_oup_portd
	ori r16,Hupe
    sts sr_oup_portd,r16
    ldi r17,1
	sts merker3,r17
    ret
;
;
end_hupe:
    lds r16,sr_oup_portd
	andi r16,(255-Hupe)
    sts sr_oup_portd,r16
    ldi r17,0
	sts merker3,r17
    ret
;-------------------------------------------------------------------------------
;	Prozessorzeit verschwenden weil er noch nicht viel zu tun hat
;   z.B. 1000 mal im Kreis laufen und trotzdem kommt er noch auf
;   ca. 2125 Zyklen pro Sec. siehe Parameter 935 bzw. rechte Zahl
;   in der oberen Zeile
;-------------------------------------------------------------------------------
;  
im_kreis_laufen:
    ldi r16,100
	ldi r17,10
im_kreis:
    nop
    nop
    nop
    dec r16
    brne im_kreis
;
    ldi r16,100
    dec r17
    brne im_kreis
;
    ret
 

.EXIT                           ;Ende des Quelltextes
